# -*- coding: iso-8859-1 -*-
# pylint: disable-msg=C0101,C0103,W0232,W0131,E0213,E0201,R0914,R0912,W0622,
# pylint: disable-msg=W0102
# pylint-version = 0.7.0
#
# Copyright 2005 Andr Malo or his licensors, as applicable
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""
TestBase class, which provides a lot of mocked modules of the standard library
"""
__author__    = "Andr Malo"
__docformat__ = "restructuredtext en"
__all__       = ['TestBase', 'Store']

# global imports
from svnmailer_test import util


class Store(object):
    """ Simple data container """

    def __init__(self, **kwargs):
        """ Initialization """
        self.__dict__.update(kwargs)


class TestBase(util.TestBase):
    """ Provides a lot of stdlib mocks (incomplete, of course) """

    def _mock_datetime(self):
        """ datetime mock """
        class datetime(object):
            def __new__(cls, **kwargs):
                if self.mock.error == 'datetime.value':
                    raise ValueError('foo')
                self.mock.result.extend(['datetime.__new__', kwargs])
                obj = object.__new__(cls)
                obj.__dict__.update(kwargs)
                return obj
            def strftime(this, format):
                if format == "%Y%m%d%H%M%S":
                    return '%04d%02d%02d%02d%02d%02d' % (
                        this.year, this.month, this.day, this.hour,
                        this.minute, this.second
                    )
                raise NotImplementedError("unknown format %s" % format)

        return ('datetime', {
            'datetime': datetime,
        })


    def _mock_errno(self):
        """ errno mock """
        errnos = {
            'ENOENT':  2,
            'ESRCH' :  3,
            'EINTR' :  4,
            'EAGAIN': 11,
            'ENOMEM': 12,
            'EEXIST': 17,
            'EPIPE' : 32,
        }
        errorcode = dict([(val, var) for var, val in errnos.items()])

        mdict = {'errorcode': errorcode}
        mdict.update(errnos)
        return ('errno', mdict)


    def _mock_httplib(self):
        """ httplib mock """
        class HTTPException(Exception):
            pass

        class HTTPConnection(object):
            def __init__(this, host):
                self.mock.result.extend(['httplib.HTTPConnection', host])
            def request(this, method, url, body = None, headers = {}):
                if self.mock.error == 'httplib.socket':
                    raise self.mock['socket'].timeout("dumdidum")
                self.mock.result.extend(['httplib.HTTPConnection.request',
                    method, url, body, headers])
            def getresponse(this):
                if self.mock.error == 'httplib.socket2':
                    raise self.mock['socket'].timeout('tralala')
                self.mock.result.append('httplib.HTTPConnection.getresponse')
                return '*someresponse'

        return ('httplib', {
            'HTTPException': HTTPException,
            'HTTPConnection': HTTPConnection,
        })


    def _mock_msvcrt(self):
        """ msvcrt mock """
        def open_osfhandle(handle, arg):
            self.mock.result.extend(['msvcrt.open_osfhandle', handle, arg])
            return handle

        return ('msvcrt', {
            'open_osfhandle': open_osfhandle,
        })


    def _mock_optparse(self):
        """ optparse mock """
        self.mock.optparse = Store(width = 60, minwidth = 60)

        class OptionParser:
            def __init__(this, *args, **kwargs):
                self.mock.result.extend([
                    'optparse.OptionParser', args, kwargs
                ])
            def parse_args(this, argv, *args, **kwargs):
                self.mock.result.extend([
                    'optparse.OptionParser.parse_args', argv, args, kwargs
                ])
                return "options", []
            def format_help(this, formatter = None):
                if self.mock.optparse.width < self.mock.optparse.minwidth:
                    raise ValueError("too small")
                self.mock.result.extend([
                    'optparse.OptionParser.format_help', formatter
                ])
                return "formatted help"

        class IndentedHelpFormatter:
            def __init__(this, *args, **kwargs):
                this._mocked = 'IndentedHelpFormatterMocked'
                self.mock.result.extend([
                    'optparse.IndentedHelpFormatter', args, kwargs
                ])

        return ('optparse', {
            'OptionParser': OptionParser,
            'IndentedHelpFormatter': IndentedHelpFormatter,
        })


    def _mock_os(self):
        """ os mock """
        errno = self._mock_errno()[1]
        self.mock.os = Store(
            curfd = 3,
            forkpids = [31415],
            exitcode = 271828,
            readbytes = '',
            listdir = {},
            path = Store(
                abspath = {},
                basename = {'/foo/bar': 'bar'},
                dirname = {},
                isdir = []
            ),
        )

        def chmod(name, mode):
            if self.mock.error == 'os.chmod':
                raise OSError(errno['ENOENT'], "doh!")
            self.mock.result.extend(['os.chmod', name, mode])

        def close(fd):
            if self.mock.error == 'os.close':
                raise OSError(errno['ENOMEM'], "out of memory...")
            self.mock.result.extend(['os.close', fd])

        def dup(fd):
            self.mock.result.extend(['os.dup', fd])
            fd = self.mock.os.curfd
            self.mock.os.curfd += 1
            return fd

        def dup2(fd, fd2):
            self.mock.result.extend(['os.dup2', fd, fd2])

        def _exit(exitcode):
            self.mock.result.extend(['os._exit', exitcode])

        def fdopen(fd, mode = 'r', bufsize = None):
            self.mock.result.extend(['os.fdopen', fd, mode, bufsize])
            return '*file%d' % fd

        def fork():
            if self.mock.error == 'os.fork':
                raise OSError(errno['ENOMEM'], "out of memory...")
            self.mock.result.append('os.fork')
            if self.mock.os.forkpids:
                return self.mock.os.forkpids.pop(0)
            return 0

        def kill(pid, signum):
            if self.mock.error == 'os.kill':
                raise OSError(errno['ENOMEM'], "out of memory...")
            elif self.mock.error == 'os.kill.gone':
                raise OSError(errno['ESRCH'], "does not exist")
            self.mock.result.extend(['os.kill', pid, signum])

        def listdir(directory):
            self.mock.result.extend(['os.listdir', directory])
            if self.mock.error == 'os.listdir':
                raise OSError(errno['ENOENT'], "Doh!")
            return self.mock.os.listdir[directory]

        def osopen(path, flags, mode):
            if self.mock.error == 'os.open.exist':
                raise OSError(errno['EEXIST'], 'abcdef')
            if self.mock.error == 'os.open':
                raise OSError(errno['ENOENT'], 'ugh')
            self.mock.result.extend(['os.open', path, flags, mode])
            fd = self.mock.os.curfd
            self.mock.os.curfd += 1
            return fd

        class Path(object):
            sep = '/'

            def abspath(this, path):
                self.mock.result.extend(['os.path.abspath', path])
                return self.mock.os.path.abspath[path]

            def basename(this, path):
                self.mock.result.extend(['os.path.basename', path])
                return self.mock.os.path.basename[path]

            def dirname(this, path):
                self.mock.result.extend(['os.path.dirname', path])
                return self.mock.os.path.dirname[path]

            def isdir(this, path):
                self.mock.result.extend(['os.path.isdir', path])
                if self.mock.error == 'os.path.isdir':
                    raise OSError(errno['ENOENT'], "Foo!")
                return path in self.mock.os.path.isdir

            def join(this, *arg):
                self.mock.result.extend(['os.path.join', arg])
                return '/'.join(arg)

        def pipe():
            self.mock.result.append('os.pipe')
            rfd = self.mock.os.curfd
            wfd = self.mock.os.curfd + 1
            self.mock.os.curfd += 2
            return (rfd, wfd)

        def popen2(cmd, mode = None, bufsize = None):
            res = self.mock.result[:]
            res.extend(['os.popen2', cmd, mode, bufsize])
            fd1, fd2 = pipe()
            self.mock.result = res
            return ("*file%d" % fd1, "*file%d" % fd2)

        def popen4(cmd, mode = None, bufsize = None):
            res = self.mock.result[:]
            res.extend(['os.popen4', cmd, mode, bufsize])
            fd1, fd2 = pipe()
            self.mock.result = res
            return "*file%d" % fd1, "*file%d" % fd2

        def read(fd, maxbytes):
            if self.mock.error == 'os.read':
                raise OSError(errno['ENOMEM'], "out of memory...")
            self.mock.result.extend(['os.read', fd, maxbytes])
            bytes = self.mock.os.readbytes[:maxbytes]
            self.mock.os.readbytes = self.mock.os.readbytes[maxbytes:]
            return bytes

        def rename(src, dest):
            if self.mock.error == 'os.rename':
                raise OSError(errno['ENOENT'], "doh!")
            self.mock.result.extend(['os.rename', src, dest])

        def setsid():
            self.mock.result.append('os.setsid')

        def system(cmd):
            self.mock.result.extend(['os.system', cmd])

        def unlink(filepath):
            if self.mock.error == 'os.unlink.notfound':
                raise OSError(errno['ENOENT'], "file not found")
            elif self.mock.error == 'os.unlink':
                raise OSError(errno['ENOMEM'], "out of memory...")
            self.mock.result.extend(['os.unlink', filepath])

        def waitpid(pid, flags):
            self.mock.result.extend(['os.waitpid', pid, flags])
            return self.mock.os.exitcode

        def write(fd, towrite):
            if self.mock.error == 'os.write':
                raise OSError(0, "some os error")
            elif self.mock.error == 'os.write.pipe':
                raise OSError(errno['EPIPE'], "EPIPE received")
            self.mock.result.extend(['os.write', fd, towrite])
            if self.mock.error == 'os.write.0':
                return 0
            elif self.mock.error == 'os.write.eintr':
                raise OSError(errno['EINTR'], "Interrupted")
            return len(towrite)

        return ('os', {
            'chmod'   : chmod,
            'close'   : close,
            'dup'     : dup,
            'dup2'    : dup2,
            '_exit'   : _exit,
            'EX_OK'   : "EX_OK",
            'EX_SOFTWARE': "EX_SOFTWARE",
            'fdopen'  : fdopen,
            'fork'    : fork,
            'kill'    : kill,
            'listdir' : listdir,
            'O_CREAT' : 64,
            'O_EXCL'  : 128,
            'O_TRUNC' : 512,
            'O_WRONLY': 1,
            'open'    : osopen,
            'path'    : Path(),
            'pipe'    : pipe,
            'popen2'  : popen2,
            'popen4'  : popen4,
            'read'    : read,
            'rename'  : rename,
            'setsid'  : setsid,
            'system'  : system,
            'unlink'  : unlink,
            'waitpid' : waitpid,
            'write'   : write,
        })


    def _mock_popen2(self):
        """ popen2 mock """
        class Popen3:
            def _run_child(this, cmd):
                self.mock.result.extend(['popen2.Popen3._run_child', cmd])
            def wait(this):
                self.mock.result.append('popen2.Popen3.wait')
                return 0

        def _cleanup():
            self.mock.result.append('popen2._cleanup')

        return ('popen2', {
            'Popen3': Popen3,
            '_cleanup': _cleanup,
            '_active': [],
        })


    def _mock_posixpath(self):
        """ posixpath mock """
        self.mock.posixpath = Store(track = True)

        def join(*arg):
            if self.mock.posixpath.track:
                self.mock.result.extend(['posixpath.join'] + list(arg))
            return '/'.join(arg)

        def split(path):
            if self.mock.posixpath.track:
                self.mock.result.extend(['posixpath.split', path])
            idx = path.rfind('/')
            return path[:idx], path[idx+1:]

        return ('posixpath', {
            'join' : join,
            'split': split,
        })


    def _mock_re(self):
        """ re mock """
        self.mock.re = Store(show_compile = False)
        import re # should be the original

        class Regex(object):
            def __init__(this, regex):
                this._regex = regex
            def match(this, tomatch):
                self.mock.result.extend(['Regex.match',
                    this._regex.pattern, tomatch])
                return this._regex.match(tomatch)
            def search(this, tosearch):
                self.mock.result.extend(['Regex.search',
                    this._regex.pattern, tosearch])
                return this._regex.search(tosearch)
            def sub(this, replace, tosub):
                self.mock.result.extend(['Regex.sub',
                    this._regex.pattern, replace, tosub])
                return this._regex.sub(replace, tosub)
            def split(this, tosplit, maxsplit = 0):
                self.mock.result.extend(['Regex.split',
                    this._regex.pattern, tosplit, maxsplit])
                return this._regex.split(tosplit, maxsplit)

        def compile(regex, flags = 0):
            if self.mock.re.show_compile:
                self.mock.result.extend(['re.compile', regex, flags])
            if isinstance(regex, Regex):
                regex = regex._regex
            rex = re.compile(regex, flags)
            return Regex(rex)

        return ('re', {
            'compile': compile,
            'X'      : re.X,
            'error'  : re.error,
        })


    def _mock_signal(self):
        """ signal mock """
        self.mock.signal = Store(signals = {})

        def signal(signum, action):
            if hasattr(action, 'func_name'):
                self.mock.signal.signals[signum] = action
                action = action.func_name
            self.mock.result.extend(['signal', signum, action])

        return ('signal', {
            'signal' : signal,
            'SIG_DFL':  0,
            'SIG_IGN':  1,
            'SIGHUP' :  1,
            'SIGINT' :  2,
            'SIGUSR2': 12,
            'SIGPIPE': 13,
            'SIGALRM': 14,
            'SIGTERM': 15,
        })


    def _mock_SimpleXMLRPCServer(self):
        """ SimpleXMLRPCServer mock """
        class Socket(object):
            def settimeout(this, timeout):
                self.mock.result.extend([
                    'SimpleXMLRPCServer.socket.settimeout', timeout
                ])

        class SimpleXMLRPCServer:
            socket = Socket()
            def __init__(this, *args, **kwargs):
                self.mock.result.extend(['SimpleXMLRPCServer', args, kwargs])
            def register_instance(this, inst):
                self.mock.result.extend([
                    'SimpleXMLRPCServer.register_instance',
                    inst.__class__.__name__
                ])
            def handle_request(this):
                self.mock.result.append('SimpleXMLRPCServer.handle_request')

        class SimpleXMLRPCRequestHandler:
            client_address = ('1.2.3.4', 79)
            def __init__(this):
                pass
            def parse_request(this):
                self.mock.result.append(
                    'SimpleXMLRPCRequestHandler.parse_request'
                )
                if self.mock.error == 'rpc.handler.parse':
                    return False
                return True
            def send_error(this, code, msg):
                self.mock.result.extend([
                    'SimpleXMLRPCRequestHandler.send_error', code, msg
                ])

        return ('SimpleXMLRPCServer', {
            'SimpleXMLRPCServer': SimpleXMLRPCServer,
            'SimpleXMLRPCRequestHandler': SimpleXMLRPCRequestHandler,
        })


    def _mock_socket(self):
        """ socket mock """
        class error(Exception):
            pass
        class timeout(error):
            pass
        class gaierror(error):
            pass
        class herror(error):
            pass

        def getaddrinfo(host, port, *arg):
            if self.mock.error == 'socket.getaddrinfo':
                raise gaierror(-3, 'goh')
            self.mock.result.extend(['socket.getaddrinfo', host, port, arg])
            if self.mock.error == 'socket.getaddrinfo.empty':
                return []
            return [
                (2, 1, 6, '', ('193.99.144.86', 80)),
                (2, 1, 6, '', ('193.99.144.85', 80))
            ]

        class Socket(object):
            mocked = '*socket.socket'
            def settimeout(this, timeout):
                self.mock.result.extend(['socket.socket.settimeout', timeout])
            def connect(this, address):
                if self.mock.error == 'socket.connect':
                    raise timeout("oops")
                self.mock.result.extend(['socket.socket.connect', address])
            def close(this):
                self.mock.result.append('socket.socket.close')

        def socket(family, socktype, proto):
            self.mock.result.extend(['socket.socket', family, socktype, proto])
            return Socket()

        return ('socket', {
            'error'      : error,
            'gaierror'   : gaierror,
            'herror'     : herror,
            'timeout'    : timeout,
            'getaddrinfo': getaddrinfo,
            'socket'     : socket,
            'EAI_AGAIN'  :  -3,
            'EAI_FAIL'   :  -4,
            'EAI_SERVICE':  -8,
            'EAI_MEMORY' : -10,
            'AF_INET'    :   2,
            'SOCK_STREAM':   1,
        })


    def _mock_subprocess(self):
        """ subprocess mock """
        class Popen(object):
            def __init__(this, *args, **kwargs):
                self.mock.result.extend(['subprocess.Popen', args, kwargs])
                this.stdin = '*stdin'
                this.stdout = '*stdout'
            def wait(this):
                self.mock.result.append('subprocess.Popen.wait')
                return 0

        return ('subprocess', {
            'Popen': Popen,
            'PIPE': '*PIPE',
            'STDOUT': '*STDOUT',
        })


    def _mock_tempfile(self):
        """ tempfile mock """
        self.mock.tempfile = Store(
            curfd = 100, names = ['tmp1', 'tmp2', 'tmp3']
        )

        def mkstemp(dir = '/tmp', prefix = '', suffix = '', text = False):
            if self.mock.error == 'tempfile.mkstemp':
                raise IOError('foo')
            self.mock.result.extend([
                'tempfile.mkstemp', dir, prefix, suffix, text
            ])
            fd = self.mock.tempfile.curfd
            self.mock.tempfile.curfd += 1
            name = prefix + self.mock.tempfile.names.pop(0) + suffix
            return (fd, dir + "/" + name)

        return ('tempfile', {
            'mkstemp': mkstemp,
        })


    def _mock_textwrap(self):
        """ textwrap mock """
        class TextWrapper(object):
            def __init__(this, **kwargs):
                self.mock.result.extend(['textwrap.TextWrapper', kwargs])
                this.__dict__.update(kwargs)
            def fill(this, text):
                parms = dict([(key, val) for key, val in this.__dict__.items()
                    if not key.startswith('_') and not callable(val)])
                self.mock.result.extend(['textwrap.TextWrapper.fill',
                    parms, text])
                return "BROKEN(%s)" % text

        return ('textwrap', {
            'TextWrapper': TextWrapper,
        })


    def _mock_time(self):
        """ time mock """
        self.mock.time = Store(curtime = float(31415))

        def time():
            self.mock.result.append('time.time')
            return self.mock.time.curtime

        def sleep(duration):
            self.mock.result.extend(['time.sleep', duration])

        def strftime(format):
            self.mock.result.extend(['time.strftime', format])
            if format == '%Y%m%d%H%M%S':
                return '20050704213710'
            raise AssertionError("Unhandled strftime format in mock")

        return ('time', {
            'time'    : time,
            'sleep'   : sleep,
            'strftime': strftime,
        })


    def _mock_urllib(self):
        """ urllib mock """
        def quote(s, safe = '/'):
            self.mock.result.extend(['urllib.quote', s, safe])
            return "quoted(%s)" % str(s)

        return ('urllib', {
            'quote': quote,
        })


    def _mock_urlparse(self):
        """ urlparse mock """
        self.mock.urlparse = Store(
            urlparse = {
                'http://opensource.perlig.de/':
                    ('http', 'opensource.perlig.de', '/', '', '', '')
            }
        )

        def urlparse(url, scheme='', allow_fragments=1):
            self.mock.result.extend(['urlparse.urlparse',
                url, scheme, allow_fragments])
            return self.mock.urlparse.urlparse[url]

        def urlunparse((scheme, netloc, url, params, query, fragment)):
            self.mock.result.extend(['urlparse.urlunparse',
                scheme, netloc, url, params, query, fragment])
            return '|'.join((scheme, netloc, url, params, query, fragment))

        return ('urlparse', {
            'urlparse': urlparse,
            'urlunparse': urlunparse,
        })


    def _mock_warnings(self):
        """ warnings mock """
        def warn(message, category = None, stacklevel = None):
            self.mock.result.extend(['warnings.warn',
                message, category, stacklevel])

        return ("warnings", {
            'warn': warn,
        })


    def _win32_mockhelper(self):
        """ mocking helper for win32 modules """
        class Handle(object):
            def __init__(this, value):
                this.value = value
            def Detach(this):
                self.mock.result.extend(['Handle.Detach', this.value])
                return this.value
            def Close(this):
                self.mock.result.extend(['Handle.Close', this.value])

        return {
            'Handle': Handle,
        }


    def _mock_win32api(self):
        """ win32api mock """
        _handle = self._win32_mockhelper()['Handle']
        def GetStdHandle(handle):
            self.mock.result.extend(['win32api.GetStdHandle', handle])
            return _handle(abs(handle + 1))

        def GetCurrentProcess():
            self.mock.result.append('win32api.GetCurrentProcess')
            return -1

        def DuplicateHandle(source, handle, target, attr1, attr2, attr3):
            self.mock.result.extend(['win32api.DuplicateHandle',
                source, handle, target, attr1, attr2, attr3
            ])
            return -handle

        return ('win32api', {
            'GetStdHandle': GetStdHandle,
            'GetCurrentProcess': GetCurrentProcess,
            'DuplicateHandle': DuplicateHandle,
            'STD_INPUT_HANDLE': -1,
            'STD_OUTPUT_HANDLE': -2,
            'STD_ERROR_HANDLE': -3,
        })


    def _mock_win32con(self):
        """ win32con mock """
        return ('win32con', {
            'DETACHED_PROCESS': 8,
            'DUPLICATE_SAME_ACCESS': 3,
        })


    def _mock_win32event(self):
        """ win32event mock """
        def WaitForSingleObject(handle, timeout):
            self.mock.result.extend(['win32event.WaitForSingleObject',
                handle.value, timeout
            ])

        return ('win32event', {
            'WaitForSingleObject': WaitForSingleObject,
            'INFINITE': -1,
        })


    def _mock_win32file(self):
        """ win32file mock """
        def CloseHandle(handle):
            self.mock.result.extend(['win32file.CloseHandle', handle.value])

        return ('win32file', {
            'CloseHandle': CloseHandle,
        })


    def _mock_win32pipe(self):
        """ win32pipe mock """
        self.mock.win32pipe = Store(curfd = 3)
        _handle = self._win32_mockhelper()['Handle']

        def CreatePipe(arg1, arg2):
            self.mock.result.extend(['win32pipe.CreatePipe', arg1, arg2])
            fd1 = self.mock.win32pipe.curfd
            self.mock.win32pipe.curfd += 1
            fd2 = self.mock.win32pipe.curfd
            self.mock.win32pipe.curfd += 1
            return _handle(fd1), _handle(fd2)

        return ('win32pipe', {
            'CreatePipe': CreatePipe,
        })


    def _mock_win32process(self):
        """ win32process mock """
        _handle = self._win32_mockhelper()['Handle']

        class STARTUPINFO(object):
            def __init__(this):
                this.dwFlags = 0
                this.hStdInput = None
                this.hStdOutput = None
                this.hStdError = None

        def CreateProcess(exe, cmd, attr1, attr2, inherit, flags, attr3, attr4,
                          start):
            self.mock.result.extend(['win32process.CreateProcess',
                exe, cmd, attr1, attr2, inherit, flags,
                attr3, attr4, start.dwFlags, start.hStdInput.value,
                start.hStdOutput.value, start.hStdError.value,
            ])
            return (_handle(100), _handle(200), _handle(3), _handle(4))

        def GetExitCodeProcess(handle):
            self.mock.result.extend(['win32process.GetExitCodeProcess',
                handle.value
            ])
            return 31415

        return ('win32process', {
            'CreateProcess': CreateProcess,
            'GetExitCodeProcess': GetExitCodeProcess,
            'STARTUPINFO': STARTUPINFO,
            'STARTF_USESTDHANDLES': 1,
        })


    def _mock_xmlrpclib(self):
        """ xmlrpclib mock """
        class Error(Exception):
            pass

        class Transport(object):
            mocked = 'xmlrpclib.Transport'
            def parse_response(this, fileobj):
                if self.mock.error == 'xmlrpclib.transport.exit':
                    raise SystemExit(1)
                elif self.mock.error == 'xmlrpclib.transport.intr':
                    raise KeyboardInterrupt()
                elif self.mock.error == 'xmlrpclib.transport.socket':
                    raise self.mock['socket'].error(0, 'buh')
                elif self.mock.error == 'xmlrpclib.transport.io':
                    raise IOError(0, 'buh')
                elif self.mock.error == 'xmlrpclib.transport.xml':
                    raise ValueError("Doh!")
                self.mock.result.extend(['xmlrpclib.Transport', fileobj.mocked])
                return tuple(fileobj.read())

        class ServerProxy(object):
            def __init__(this, uri, transport = Transport):
                self.mock.result.extend(['ServerProxy', uri, transport.mocked])

        def Binary(nonbinary):
            self.mock.result.extend(['xmlrpclib.Binary', nonbinary])
            return nonbinary.encode('base64')

        return ('xmlrpclib', {
            'Error'      : Error,
            'ServerProxy': ServerProxy,
            'Transport'  : Transport,
            'Binary'     : Binary,
            'True'       : 'xmlrpclib.TRUE',
        })
