# -*- coding: iso-8859-1 -*-
# pylint: disable-msg=E0201,W0131,E0213,W0232
# pylint-version = 0.7.0
#
# Copyright 2005 Andr Malo or his licensors, as applicable
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""
Testcases for svnmailer.browser.websvn

:var _tests: The test classes
:type _tests: ``list``
"""
__author__    = "Andr Malo"
__docformat__ = "restructuredtext en"
__all__       = ['getSuite']

from svnmailer_test import base
_tests = []


#### Test Bases                                     ####
########################################################
class TestBase(base.TestBase):
    """ Base class for websvn generator tests """
    LOAD_MODULES = (('_websvn', 'svnmailer.browser.websvn'),)

    def getInitialMocks(self):
        """ Returns the initially mocked modules """
        return [
            self._mock_svnmailer_browser_base(),
            self._mock_posixpath(),
        ]


#### Test Cases                                     ####
########################################################
class ErrorTestCase(TestBase):
    """ svnmailer.browser.websvn.Error """

    def test00general(self):
        """ Error is a public browser.Error """
        from svnmailer import browser
        expected = browser.Error
        result = self._websvn.Error

        self.failUnless(issubclass(result, expected),
            "Is not an %r, but %r" % (expected, result))
        self.failUnless(result.__name__ in self.public(self._websvn),
            "Is not public")

_tests.append(ErrorTestCase)


class MissingPHPFileErrorTestCase(TestBase):
    """ svnmailer.browser.websvn.MissingPHPFileError """

    def test00general(self):
        """ MissingPHPFileError is a public Error """
        expected = self._websvn.Error
        result = self._websvn.MissingPHPFileError

        self.failUnless(issubclass(result, expected),
            "Is not an %r, but %r" % (expected, result))
        self.failUnless(result.__name__ in self.public(self._websvn),
            "Is not public")

_tests.append(MissingPHPFileErrorTestCase)


class GeneratorTestCase(TestBase):
    """ svnmailer.browser.websvn.Generator """
    PREFIX = "Generator"

    def test00general(self):
        """ Generator is a public _base.BaseGenerator """
        expected = self.mock['svnmailer.browser._base'].BaseGenerator
        result = self._websvn.Generator

        self.failUnless(issubclass(result, expected),
            "Is not an %r, but %r" % (expected, result))
        self.failUnless(result.__name__ in self.public(self._websvn),
            "Is not public")

    def test01createTemplate(self):
        """ ._createTemplate selects the path info template """
        class MyGenerator(self._websvn.Generator):
            def _getPiTemplate(this, url):
                self.mock.result.append('path info')
                return url, {}
        inst = MyGenerator()
        self.mock.result.append(inst._createTemplate('foo', 'bar')._mocked)

        self.expect(
            'browser._base.BaseGenerator._createParsedUrl', 'foo',
            'path info',
            'browser._base.Template.fromTemplates', 'foo', {},
            'Template'
        )

    def test02createTemplate(self):
        """ ._createTemplate selects the non path info template """
        self.mock.mailer_browser_base.querykeys.append(u'repname')
        class MyGenerator(self._websvn.Generator):
            def _getNopiTemplate(this, url):
                self.mock.result.append('no path info')
                return url, {}
        inst = MyGenerator()
        self.mock.result.append(inst._createTemplate('foo', 'bar')._mocked)

        self.expect(
            'browser._base.BaseGenerator._createParsedUrl', 'foo',
            'no path info',
            'browser._base.Template.fromTemplates', 'foo', {},
            'Template'
        )

    def test03createTemplate(self):
        """ ._createTemplate passes AttibuteError """
        self.mock.error = 'browser.template.attr'
        inst = self._websvn.Generator()

        self.failUnlessRaises(AttributeError,
            inst._createTemplate, 'foo', 'bar')

    def test04createTemplate(self):
        """ ._createTemplate passes MissingPHPFileError """
        self.mock.mailer_browser_base.querykeys.append(u'repname')
        class MyGenerator(self._websvn.Generator):
            def _getNopiTemplate(this, url):
                raise self._websvn.MissingPHPFileError()
        inst = MyGenerator()
        self.failUnlessRaises(self._websvn.MissingPHPFileError,
            inst._createTemplate, 'foo', 'bar')

    def test05getNopiTemplate(self):
        """ ._getNopiTemplate returns Template parameters """
        inst = self._websvn.Generator()
        url = self.mock['svnmailer.browser._base'].ParsedUrl('foo')
        self.mock.result.append(inst._getNopiTemplate(url))

        self.expect(
            'posixpath.split', '/foo/bar',
            (url, {
                'revision': (u'listing.php', {
                    'set': [(u'sc', u'1'), (u'rev', 'REVISION')],
                    'remove': [u'path']}),
                'deleted': (u'filedetails.php', {
                    'set': [(u'rev', 'BASE_REVISION'),
                        (u'path', u'/BASE_PATH')],
                    'remove': [u'sc']}),
                'deleted_dir': (u'listing.php', {
                    'set': [(u'rev', 'BASE_REVISION'),
                        (u'path', u'/BASE_PATH/')],
                    'remove': [u'sc']}),
                'added': (u'filedetails.php', {
                    'set': [(u'rev', 'REVISION'), (u'path', u'/PATH')],
                    'remove': [u'sc']}),
                'added_dir': (u'listing.php', {
                    'set': [(u'rev', 'REVISION'), (u'path', u'/PATH/')],
                    'remove': [u'sc']}),
                'copied': (u'diff.php', {
                    'set': [(u'rev', 'REVISION'), (u'path', u'/PATH')],
                    'remove': [u'sc']}),
                'modified': (u'diff.php', {
                    'set': [(u'rev', 'REVISION'), (u'path', u'/PATH')],
                    'remove': [u'sc']}),
            }),
        )

    def test06getNopiTemplate(self):
        """ ._getNopiTemplate raises MissingPHPFileError """
        self.mock.mailer_browser_base.urlpath = '/foo/bar/'
        inst = self._websvn.Generator()
        url = self.mock['svnmailer.browser._base'].ParsedUrl('foo')

        self.failUnlessRaises(self._websvn.MissingPHPFileError,
            inst._getNopiTemplate, url)

    def test07getPiTemplate(self):
        """ ._getPiTemplate returns Template parameters """
        self.mock.mailer_browser_base.urlpath = '/foo/bar/'
        inst = self._websvn.Generator()
        url = self.mock['svnmailer.browser._base'].ParsedUrl('foo')
        self.mock.result.append(inst._getPiTemplate(url))

        self.expect(
            'browser._base.ParsedQuery.remove', [u'path'],
            (url, {
                'revision': (None, {
                    'set': [(u'sc', u'1'), (u'rev', 'REVISION')]}),
                'deleted': ('BASE_PATH', {
                    'set': [(u'rev', 'BASE_REVISION'), (u'op', u'file')],
                    'remove': [u'sc']}),
                'deleted_dir': (u'BASE_PATH/', {
                    'set': [(u'rev', 'BASE_REVISION'), (u'op', u'dir')],
                    'remove': [u'sc']}),
                'added': ('PATH', {
                    'set': [(u'rev', 'REVISION'), (u'op', u'file')],
                    'remove': [u'sc']}),
                'added_dir': (u'PATH/', {
                    'set': [(u'rev', 'REVISION'), (u'op', u'dir')],
                    'remove': [u'sc']}),
                'copied': ('PATH', {
                    'set': [(u'rev', 'REVISION'), (u'op', u'diff')],
                    'remove': [u'sc']}),
                'modified': ('PATH', {
                    'set': [(u'rev', 'REVISION'), (u'op', u'diff')],
                    'remove': [u'sc']}),
            }),
        )

_tests.append(GeneratorTestCase)


#### Glue code                                      ####
########################################################
__all__.extend(_tests)

def getSuite():
    """ Returns the test suite for this module """
    return [_test.__name__ for _test in _tests]
