# -*- coding: iso-8859-1 -*-
# pylint: disable-msg=E0201,W0131,C0103,W0703,W0612,W0232,E0213,C0321,R0914
# pylint-version = 0.7.0
#
# Copyright 2004-2005 Andr Malo or his licensors, as applicable
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""
Testcases for svnmailer.notifier._base

:var _tests: The test classes
:type _tests: ``list``
"""
__author__    = "Andr Malo"
__docformat__ = "restructuredtext en"
__all__       = ['getSuite']

import unittest
_tests = []


class importTestCase(unittest.TestCase):
    """ Importing svnmailer.notifier._base """

    def testImport(self):
        """ from svnmailer.notifier import _base """
        try:
            from svnmailer.notifier import _base
        except Exception, exc:
            self.fail(
                "svnmailer.notifier._base could not be imported: %s" %
                str(exc)
            )

_tests.append(importTestCase)


class BaseNotifierTestCase(unittest.TestCase):
    """ svnmailer.notifier._base.BaseNotifier """

    def setUp(self):
        """ Create test environment """
        from svnmailer.notifier import _base
        def reorder(url):
            """ reorder query string """
            if url is None:
                return None
            if '?' not in url:
                return url
            path, query = url.split('?', 1)
            qlist = query.split('&')
            qlist.sort()
            return '?'.join([path, '&'.join(qlist)])

        self._base = _base
        self._settings = DummyContainer(
            runtime = DummyContainer(),
            general = DummyContainer(),
            groups  = []
        )
        self._config = DummyContainer()
        self._reorder = reorder

    def test00getAuthor(self):
        """ getAuthor returns the proper author """
        tests = ( (1, 'authorA'), (2, 'authorB'), )

        runtime = self._settings.runtime
        runtime._repos = DummyRepository(author = dict(tests))

        mybase = self._base.BaseNotifier(self._settings, [])
        for runtime.revision, expected in tests:
            result = mybase.getAuthor()
            self.failUnlessEqual(expected, result,
                "Rev %d doesn't return the expected author (%r), but %r" %
                (runtime.revision, expected, result)
            )

    def test01getTime(self):
        """ getTime returns the proper time """
        tests = ( (1, 10), (2, 20), )

        runtime = self._settings.runtime
        runtime._repos = DummyRepository(time = dict(tests))

        mybase = self._base.BaseNotifier(self._settings, [])
        for runtime.revision, expected in tests:
            result = mybase.getTime()
            self.failUnlessEqual(expected, result,
                "Rev %d doesn't return the expected time (%r), but %r" %
                (runtime.revision, expected, result)
            )

    def test02getLog(self):
        """ getLog returns the proper log entry """
        tests = ( (1, "log"), (2, "otherlog"), )

        runtime = self._settings.runtime
        runtime._repos = DummyRepository(log = dict(tests))

        mybase = self._base.BaseNotifier(self._settings, [])
        for runtime.revision, expected in tests:
            result = mybase.getLog()
            self.failUnlessEqual(expected, result,
                "Rev %d doesn't return the expected log (%r), but %r" %
                (runtime.revision, expected, result)
            )

    def test03getUrl(self):
        """ getUrl returns the proper URL or None """
        tests = (
            ("baseA?rev=1&view=rev", "baseA", 1),
            ("baseB?rev=2&view=rev", "baseB/", 2),
            ("baseC?rev=2&root=x&view=rev", "baseC/?root=x", 2),
            (None, None, 2),
            (None, None, 3),
        )

        runtime = self._settings.runtime
        mybase = self._base.BaseNotifier(self._settings, [])
        self._config = DummyContainer()
        for expected, base, runtime.revision in tests:
            if base:
                self._config.browser_base_url = "viewcvs %s" % base
            else:
                self._config.browser_base_url = base
            self._config._subdict = {u"revision": u"%d" % runtime.revision}
            result = self._reorder(mybase.getUrl(self._config))
            self.failUnlessEqual(expected, result,
                "Rev %d doesn't return the expected URL (%r), but %r" %
                (runtime.revision, expected, result)
            )

    def test04getDiffer(self):
        """ getDiffer returns the proper differ """
        from svnmailer import differ

        tests = (
            (differ.InternalDiffer, None),
            (differ.ExternalDiffer, ('diff',)),
        )

        general = self._settings.general
        mybase = self._base.BaseNotifier(self._settings, [])
        for expected, general.diff_command in tests:
            result = mybase.getDiffer(general.diff_command)
            self.failUnless(
                isinstance(result, expected),
                "Diff command %r doesn't give the proper differ (%r), but %r" %
                (general.diff_command, expected, result)
            )

    def test05getTempFile(self):
        """ getTempFile returns a proper tempfile object """
        from svnmailer import util

        mybase = self._base.BaseNotifier(self._settings, [])
        expected = util.TempFile
        result = mybase.getTempFile()
        self.failUnless(
            isinstance(result, expected),
            "Doesn't returned expected type (%r), but %r" % (expected, result)
        )

    def test06getTempDir(self):
        """ getTempDir returns the configured tempdir or None """
        mybase = self._base.BaseNotifier(self._settings, [])
        self._settings._charset_ = 'utf-8'
        for expected in ['/tmpdir', None]:
            self._settings.general.tempdir = expected
            result = mybase.getTempDir()
            self.failUnlessEqual(expected, result,
                "Expected %r, got %r" % (expected, result)
            )

    def test07getDiffTokens(self):
        """ getDiffTokens works correct """
        mybase = self._base.BaseNotifier(self._settings, [])
        ADD = mybase.ADD; DELETE = mybase.DELETE; COPY = mybase.COPY
        MODIFY = mybase.MODIFY; PROPCHANGE = mybase.PROPCHANGE

        tests = (
            ([ADD, COPY, DELETE, MODIFY, PROPCHANGE], None),
            ([], []),
            ([ADD, COPY, DELETE, MODIFY, PROPCHANGE], "prochange"),
            ([ADD, DELETE],        "add delete"),
            ([ADD, DELETE],        "add delete modifi"),
            ([ADD, COPY, MODIFY],  "modify copy add"),
            ([],                   "none"),
            ([DELETE, PROPCHANGE], "propchange delete"),
        )

        for expected, gendiff in tests:
            if gendiff and not isinstance(gendiff, list):
                gendiff = gendiff.decode().split()
            self._config.generate_diffs = gendiff
            tokens, funcs = mybase.getDiffTokens(self._config)
            tokens.sort()
            self.failUnlessEqual(tokens, expected,
                "Doesn't return expected tokens (%r), but %r" %
                (expected, tokens)
            )

    def test08dumpContent(self):
        """ dumpContent dumps the proper content """
        tests = dict((
            (1, ("A", -1, None,    "path1", "content one")),
            (2, ("C",  1, "path1", "path2", "content one modified\n")),
            (3, ("M",  2, "path2", "path2", "content two\n")),
            (4, ("D",  3, "path2", "path2", "")),
        ))

        class myBase(self._base.BaseNotifier):
            def getTempFile(this):
                import cStringIO
                class myFile(DummyContainer):
                    def close(that): pass
                return myFile(fp = cStringIO.StringIO())

        runtime = self._settings.runtime
        runtime._repos = DummyRepository(content = tests)
        mybase = myBase(self._settings, [])
        for rev, (action, base_rev, base_path, path, content) in tests.items():
            change = DummyChange(
                path = path, _action = action, revision = rev,
                _basepath = base_path, _baserev = base_rev,
            )
            expected = (tests.get(base_rev, [''])[-1:][0], content)
            file1, file2 = mybase.dumpContent(change)[:2]
            result = (file1.fp.getvalue(), file2.fp.getvalue())
            file1.fp.close(); file2.fp.close()
            self.failUnlessEqual(expected, result,
                "Rev %d doesn't return the expected dumps (%r), but %r" %
                (rev, expected, result)
            )

    def tes09getContentEncodingProperty(self):
        """ getContentEncodingProperty returns the property content or None """
        tests = (
            ('foo', '/', 1),
            ('bar', '/', 2),
            (None,  '/', 3),
        )
        props = {
            ('svnmailer:content-charset', '/', 1): 'foo',
            ('svnmailer:content-charset', '/', 2): 'bar',
        }
        self._settings.runtime._repos = DummyRepository(props = props)
        mybase = self._base.BaseNotifier(self._settings, [])

        for expected, path, revision in tests:
            result = mybase.getContentEncodingProperty(path, revision)
            self.failUnlessEqual(expected, result,
                "%r: expected %r, got %r" %
                ((path, revision), expected, result)
            )

    def test10getContentDiffUrl(self):
        """ getContentDiffUrl returns the proper URL or None """
        tests = (
            (1, ("A", -1, None,    "path1", "baseA/",
                "baseA/path1?rev=1&view=auto"
            )),
            (2, ("C",  1, "path1", "path2", "baseB",
                "baseB/path2?p1=path1&p2=path2&r1=1&r2=2&rev=2&view=diff"
            )),
            (3, ("M",  2, "path2", "path2", "baseC/",
                "baseC/path2?r1=2&r2=3&rev=3&view=diff"
            )),
            (4, ("M",  3, "path2", "path2", None, None)),
            (5, ("D",  4, "path2", "path2", "baseB",
                "baseB/path2?rev=4&view=auto"
            )),
        )

        mybase = self._base.BaseNotifier(self._settings, [])
        for rev, (action, base_rev, base_path, path, base_url, exp) in tests:
            change = DummyChange(
                path = path, _action = action, revision = rev,
                _basepath = base_path, _baserev = base_rev,
            )
            config = DummyContainer(
                browser_base_url = [
                    base_url, "viewcvs %s" % base_url][bool(base_url)
            ])
            config._subdict = {u"revision": u"%d" % rev}
            result = self._reorder(mybase.getContentDiffUrl(config, change))
            self.failUnlessEqual(exp, result,
                "Rev %d doesn't return the expected URL (%r), but %r" %
                (rev, exp, result)
            )

    def test11isBinaryProperty(self):
        """ isBinaryProperty makes reasonable decisions """
        tests = (
            (True,  ("\0",     "x")),
            (True,  ("\1",     "y")),
            (True,  ("x",      "\0")),
            (False, ("xxxx",   "yyyy")),
            (True,  ("a",      "a" * 500)),
            (False, ("xxxx",   "\n\f\t ")),
            (True , ("xxxx\5", None)),
            (False, (None,     "yyyy")),
        )

        mybase = self._base.BaseNotifier(self._settings, [])
        for expected, values in tests:
            result = mybase.isBinaryProperty(values)
            self.failUnlessEqual(expected, result,
                "%r are not considered %r, but %r" % (values, expected, result)
            )

    def test12isBinaryProperty(self):
        """ isOneLineProperty makes reasonable decisions """
        tests = (
            (True,  ("x", "x")),
            (False, ("x", "x\n")),
            (False, ("x", "x" * 80)),
        )

        mybase = self._base.BaseNotifier(self._settings, [])
        for expected, (name, value) in tests:
            result = mybase.isOneLineProperty(name, value)
            self.failUnlessEqual(expected, result,
                "%r are not considered %r, but %r" %
                ((name, value), expected, result)
            )

    def test13getContentDiffAction(self):
        """ getContentDiffAction returns the correct token """
        mybase = self._base.BaseNotifier(self._settings, [])

        tests = (
            (mybase.ADD,    "A"),
            (mybase.DELETE, "D"),
            (mybase.COPY,   "C"),
            (None,          "M"),
            (mybase.MODIFY, "MC"),
        )

        for expected, action in tests:
            change = DummyChange(_action = action)
            result = mybase.getContentDiffAction(change)
            self.failUnlessEqual(expected, result,
                "Doesn't return expected token (%r), but %r" %
                (expected, result)
            )

    def test14getPropertyDiffAction(self):
        """ getPropertyDiffAction returns the correct token """
        mybase = self._base.BaseNotifier(self._settings, [])
        tests = (
            (mybase.ADD,    (None, "x")),
            (mybase.DELETE, ("x",  None)),
            (mybase.MODIFY, ("x",  "y")),
        )

        for expected, values in tests:
            result = mybase.getPropertyDiffAction(values)
            self.failUnlessEqual(expected, result,
                "Doesn't return expected token (%r), but %r" %
                (expected, result)
            )

_tests.append(BaseNotifierTestCase)


# dummy objects
# =============
class DummyContainer(object):
    def __init__(self, **kwargs):
        self._dict = kwargs
        self._subdict = {}
    def __getattr__(self, name):
        return self._dict.get(name)
    def __call__(self, arg):
        if arg == 'subst':
            return self._subdict
        return None

class DummyRepository(object):
    def __init__(self, **kwargs):
        self._dict = kwargs
        self.path = "unused"
    def getRevisionAuthor(self, revision):
        return self._dict['author'].get(revision)
    def getRevisionTime(self, revision):
        return self._dict['time'].get(revision)
    def getRevisionLog(self, revision):
        return self._dict['log'].get(revision)
    def dumpPathContent(self, fp, path, revision):
        desc = self._dict['content'][revision]
        if path == desc[3]: fp.write(desc[4])
        else: fp.write("path mismatch (%r <-> %r)" % (path, desc[3]))
    def getPathProperty(self, name, path, revision):
        return self._dict['props'].get((name, path, revision))

class DummyChange(DummyContainer):
    def wasAdded(self):
        return self._action == "A" or self._action == "C"
    def wasDeleted(self):
        return self._action == "D"
    def wasCopied(self):
        return self._action == "C"
    def wasModified(self):
        return self._action.startswith("M")
    def hasContentChanges(self):
        return self._action == "MC"
    def getBaseRevision(self):
        return self._baserev
    def getBasePath(self):
        return self._basepath
    def isDirectory(self):
        return self._is_directory


#### Glue code                                      ####
########################################################
__all__.extend(_tests)

def getSuite():
    """ Returns the test suite for this module """
    return [_test.__name__ for _test in _tests]
