# -*- coding: iso-8859-1 -*-
# pylint: disable-msg=E0201,W0232,W0131,E0213,F0203,W0613,W0302
# pylint-version = 0.7.0
#
# Copyright 2005 Andr Malo or his licensors, as applicable
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""
Testcases for svnmailer.settings._accessors

:var _tests: The test classes
:type _tests: ``list``
"""
__author__    = "Andr Malo"
__docformat__ = "restructuredtext en"
__all__       = ['getSuite']

from svnmailer_test import base
_tests = []


#### Test Bases                                     ####
########################################################
class TestBase(base.TestBase):
    """ Base class for settings._accessors tests """
    LOAD_MODULES = (('_acc', 'svnmailer.settings._accessors'),)

    def getInitialMocks(self):
        """ Returns the initially mocked modules """
        return [
            self._mock_svnmailer_util(),
            self._mock_svnmailer_settings_base(),
            self._mock_re(),
        ]


#### Test Cases                                     ####
########################################################
class UnicodeMemberTestCase(TestBase):
    """ svnmailer.settings._accessors.UnicodeMember """
    PREFIX = "UnicodeMember"

    def setUp(self):
        """ Extends test environment """
        super(UnicodeMemberTestCase, self).setUp()
        self._descriptor = self._acc.UnicodeMember()

    def test00general(self):
        """ UnicodeMember is a public _base.BasePostmapMember """
        expected = self.mock['svnmailer.settings._base'].BasePostmapMember
        result = self._acc.UnicodeMember

        self.failUnless(issubclass(result, expected),
            "Is not an %r, but %r" % (expected, result))
        self.failUnless(result.__name__ in self.public(self._acc),
            "Is not public")

    def test01doTransform(self):
        """ .doTransform passes unicode objects """
        self.mock.result.append(self._descriptor.doTransform(u"Andr\xe9"))

        self.failUnless(isinstance(self.mock.result[-1], unicode), "No unicode")
        self.expect(u'Andr\xe9')

    def test02doTransform(self):
        """ .doTransform decodes string input """
        self._descriptor.CHARSET = 'iso-8859-1'
        self.mock.result.append(self._descriptor.doTransform("Andr\xe9"))

        self.failUnless(isinstance(self.mock.result[-1], unicode), "No unicode")
        self.expect(u'Andr\xe9')

    def test03doTransform(self):
        """ .doTransform rejects invalid types """
        self.failUnlessRaises(TypeError, self._descriptor.doTransform, self)

    def test04doTransform(self):
        """ .doTransform raises UnicodeError """
        self._descriptor.CHARSET = 'utf-8'

        self.failUnlessRaises(UnicodeError,
            self._descriptor.doTransform, "Andr\xe9")

    def test05doSubstitute(self):
        """ .doSubstitute substitutes format strings """
        self.mock.result.append(
            self._descriptor.doSubstitute(u"%(x)sbar", {'x': u'foo'})
        )

        self.expect('util.substitute', u'%(x)sbar', {'x': u'foo'}, u'foobar')

    def test06doPostmap(self):
        """ .doPostmap transforms the mapped value """
        class MyMember(self._acc.UnicodeMember):
            def transform(this, value):
                self.mock.result.extend(['transform', value])
                return "transformed(%s)" % value

        def mapper(value):
            self.mock.result.extend(['mapper', value])
            return "mapped(%s)" % value
        inst = MyMember()
        inst.mapper = mapper
        self.mock.result.append(inst.doPostmap("foo"))

        self.expect(
            'mapper', 'foo',
            'transform', 'mapped(foo)',
            'transformed(mapped(foo))'
        )

    def test07doPostmap(self):
        """ .doPostmap passes TypeError """
        class MyMember(self._acc.UnicodeMember):
            def transform(this, value):
                raise TypeError()

        def mapper(value):
            return value
        inst = MyMember()
        inst.mapper = mapper
        
        self.failUnlessRaises(TypeError, inst.doPostmap, "foo")

    def test08doPostmap(self):
        """ .doPostmap passes UnicodeError """
        class MyMember(self._acc.UnicodeMember):
            def transform(this, value):
                raise UnicodeError()

        def mapper(value):
            return value
        inst = MyMember()
        inst.mapper = mapper
        
        self.failUnlessRaises(UnicodeError, inst.doPostmap, "foo")

_tests.append(UnicodeMemberTestCase)


class StringMemberTestCase(TestBase):
    """ svnmailer.settings._accessors.StringMember """
    PREFIX = "StringMember"

    def test00general(self):
        """ StringMember is a public _base.BaseMember """
        expected = self.mock['svnmailer.settings._base'].BaseMember
        result = self._acc.StringMember

        self.failUnless(issubclass(result, expected),
            "Is not an %r, but %r" % (expected, result))
        self.failUnless(result.__name__ in self.public(self._acc),
            "Is not public")

    def test01doTransform(self):
        """ .doTransform returns the string form of the input """
        class Foo(object):
            def __str__(this):
                return "bar"
        inst = self._acc.StringMember()
        self.mock.result.append(inst.doTransform(Foo()))

        self.expect("bar")

_tests.append(StringMemberTestCase)


class IntegerMemberTestCase(TestBase):
    """ svnmailer.settings._accessors.IntegerMember """
    PREFIX = "IntegerMember"

    def test00general(self):
        """ IntegerMember is a public _base.BaseMember """
        expected = self.mock['svnmailer.settings._base'].BaseMember
        result = self._acc.IntegerMember

        self.failUnless(issubclass(result, expected),
            "Is not an %r, but %r" % (expected, result))
        self.failUnless(result.__name__ in self.public(self._acc),
            "Is not public")

    def test01doTransform(self):
        """ .doTransform passes integer numbers """
        inst = self._acc.IntegerMember()
        self.mock.result.append(inst.doTransform(11))

        self.expect(11)

    def test02doTransform(self):
        """ .doTransform converts to integers """
        inst = self._acc.IntegerMember()
        self.mock.result.append(inst.doTransform("10"))

        self.expect(10)

    def test03doTransform(self):
        """ .doTransform raises TypeError """
        inst = self._acc.IntegerMember()

        self.failUnlessRaises(TypeError, inst.doTransform, None)

    def test04doTransform(self):
        """ .doTransform raises ValueError """
        inst = self._acc.IntegerMember()

        self.failUnlessRaises(ValueError, inst.doTransform, "a")

_tests.append(IntegerMemberTestCase)


class BooleanMemberTestCase(TestBase):
    """ svnmailer.settings._accessors.BooleanMember """
    PREFIX = "BooleanMember"

    def test00general(self):
        """ BooleanMember is a public _base.BaseMember """
        expected = self.mock['svnmailer.settings._base'].BaseMember
        result = self._acc.BooleanMember

        self.failUnless(issubclass(result, expected),
            "Is not an %r, but %r" % (expected, result))
        self.failUnless(result.__name__ in self.public(self._acc),
            "Is not public")

    def test01doTransform(self):
        """ .doTransform returns the boolean value of the input """
        class Foo(object):
            def __nonzero__(this):
                return False
        inst = self._acc.BooleanMember()
        self.mock.result.append(inst.doTransform(Foo()))

        self.expect(False)

_tests.append(BooleanMemberTestCase)


class HumanBooleanMemberTestCase(TestBase):
    """ svnmailer.settings._accessors.HumanBooleanMember """
    PREFIX = "HumanBooleanMember"

    def test00general(self):
        """ HumanBooleanMember is a public _base.BaseMember """
        expected = self.mock['svnmailer.settings._base'].BaseMember
        result = self._acc.HumanBooleanMember

        self.failUnless(issubclass(result, expected),
            "Is not an %r, but %r" % (expected, result))
        self.failUnless(result.__name__ in self.public(self._acc),
            "Is not public")

    def test01initial(self):
        """ ._TRUE is properly filled initially """
        self.mock.result = [self._acc.HumanBooleanMember._TRUE]

        self.expect(('1', 'yes', 'on', 'true'))

    def test02initial(self):
        """ ._FALSE is properly filled initially """
        self.mock.result = [self._acc.HumanBooleanMember._FALSE]

        self.expect(('', '0', 'no', 'off', 'false', 'none'))

    def test03init(self):
        """ .init initializes the super class and ._human """
        inst = self._acc.HumanBooleanMember()
        inst.init()
        self.mock.result.append(inst._human)

        self.expect(
            'BaseMember.init',
            {'': False, 'on': True, 'none': False, 'false': False, 'no': False,
                '1': True, '0': False, 'off': False, 'yes': True, 'true': True}
        )

    def test04doTransform(self):
        """ .doTransform returns the proper boolean value """
        class MyMember(self._acc.HumanBooleanMember):
            _TRUE = ('a',)
            _FALSE = ('b',)
        inst = MyMember()
        inst.init()
        self.mock.result.extend([
            inst.doTransform(inp) for inp in list('abAB')])

        self.expect('BaseMember.init', True, False, True, False)

    def test05doTransform(self):
        """ .doTransform raises ValueError """
        class MyMember(self._acc.HumanBooleanMember):
            _TRUE = ('a',)
            _FALSE = ('b',)
        inst = MyMember()
        inst.init()

        self.failUnlessRaises(ValueError, inst.doTransform, 'c')
        self.failUnlessRaises(ValueError, inst.doTransform, 'C')

_tests.append(HumanBooleanMemberTestCase)


class RegexMemberTestCase(TestBase):
    """ svnmailer.settings._accessors.RegexMember """
    PREFIX = "RegexMember"

    def setUp(self):
        """ Extends test environment """
        super(RegexMemberTestCase, self).setUp()

        class MyMember(self._acc.RegexMember):
            def __init__(this):
                this.CHARSET = "iso-8859-1"
                this._flags = 0

        self._mydescriptor = MyMember
        
    def test00general(self):
        """ RegexMember is a public _base.BaseMember """
        expected = self.mock['svnmailer.settings._base'].BasePremapMember
        result = self._acc.RegexMember

        self.failUnless(issubclass(result, expected),
            "Is not an %r, but %r" % (expected, result))
        self.failUnless(result.__name__ in self.public(self._acc),
            "Is not public")

    def test01init(self):
        """ .init initializes superclass and ._flags """
        inst = self._acc.RegexMember()
        inst.param = {'flags': 1}
        inst.init()
        self.mock.result.append(inst._flags)
        self.expect('BasePremapMember.init', 1)

        inst.param = {}
        self.mock.result = []
        inst.init()
        self.mock.result.append(inst._flags)
        self.expect('BasePremapMember.init', 0)

    def test02doTransform(self):
        """ .doTransform compiles unicode strings """
        self.mock.re.show_compile = True
        inst = self._mydescriptor()
        self.mock.result.append(inst.doTransform(u'Andr\xe9')._regex.pattern)

        self.failUnless(isinstance(self.mock.result[-1], unicode), "No unicode")
        self.expect('re.compile', u'Andr\xe9', 0, u'Andr\xe9')

    def test03doTransform(self):
        """ .doTransform compiles string input as unicode """
        self.mock.re.show_compile = True
        inst = self._mydescriptor()
        self.mock.result.append(inst.doTransform('Andr\xe9')._regex.pattern)

        self.failUnless(isinstance(self.mock.result[-1], unicode), "No unicode")
        self.expect('re.compile', u'Andr\xe9', 0, u'Andr\xe9')

    def test04doTransform(self):
        """ .doTransform raises TypeError """
        inst = self._mydescriptor()

        self.failUnlessRaises(TypeError, inst.doTransform, None)

    def test05doTransform(self):
        """ .doTransform raises ValueError """
        inst = self._mydescriptor()

        self.failUnlessRaises(ValueError, inst.doTransform, '*')

    def test06doTransform(self):
        """ .doTransform raises UnicodeError """
        inst = self._mydescriptor()
        inst.CHARSET = "utf-8"

        self.failUnlessRaises(UnicodeError, inst.doTransform, '\xe9')

_tests.append(RegexMemberTestCase)


class TokenMemberTestCase(TestBase):
    """ svnmailer.settings._accessors.TokenMember """
    PREFIX = "TokenMember"

    def setUp(self):
        """ Extends test environment """
        super(TokenMemberTestCase, self).setUp()
        class MyMember(self._acc.TokenMember):
            def __init__(this):
                this._allowed = None
                this.CHARSET = 'iso-8859-1'

        self._mydescriptor = MyMember

    def test00general(self):
        """ TokenMember is a public _base.BasePremapMember """
        expected = self.mock['svnmailer.settings._base'].BasePremapMember
        result = self._acc.TokenMember

        self.failUnless(issubclass(result, expected),
            "Is not an %r, but %r" % (expected, result))
        self.failUnless(result.__name__ in self.public(self._acc),
            "Is not public")

    def test01init(self):
        """ .init initializes superclass and ._allowed """
        inst = self._acc.TokenMember()
        inst.param = {'allowed': (u'a', u'B')}
        inst.init()
        self.mock.result.append(inst._allowed)
        self.expect('BasePremapMember.init', {u'a': None, u'b': None})

        self.mock.result = []
        inst.param = {}
        inst.init()
        self.mock.result.append(inst._allowed)
        self.expect('BasePremapMember.init', None)

    def test02doTransform(self):
        """ .doTransform returns unicode input lowercased """
        self.mock.result.append(self._mydescriptor().doTransform(u"Andr\xc9"))

        self.expect(u'andr\xe9')

    def test03doTransform(self):
        """ .doTransform decodes string input """
        self.mock.result.append(self._mydescriptor().doTransform("Andr\xc9"))

        self.expect(u'andr\xe9')

    def test04doTransform(self):
        """ .doTransform passes allowed tokens """
        inst = self._mydescriptor()
        inst._allowed = dict.fromkeys((u'a', u'b'))
        self.mock.result.append(inst.doTransform('A'))
        self.mock.result.append(inst.doTransform('B'))

        self.expect(u'a', u'b')

    def test05doTransform(self):
        """ .doTransform rejects not-allowed tokens """
        inst = self._mydescriptor()
        inst._allowed = dict.fromkeys((u'a', u'b'))

        self.failUnlessRaises(ValueError, inst.doTransform, 'C')

    def test06doTransform(self):
        """ .doTransform rejects invalid types """
        self.failUnlessRaises(TypeError, self._mydescriptor().doTransform, self)

    def test07doTransform(self):
        """ .doTransform raises UnicodeError """
        inst = self._mydescriptor()
        inst.CHARSET = "utf-8"

        self.failUnlessRaises(UnicodeError, inst.doTransform, "\xc9")

_tests.append(TokenMemberTestCase)


class TokenListTestCase(TestBase):
    """ svnmailer.settings._accessors.TokenList """
    PREFIX = "TokenList"

    def test00general(self):
        """ TokenList is a private tuple """
        expected = tuple
        result = self._acc.TokenList

        self.failUnless(issubclass(result, expected),
            "Is not an %r, but %r" % (expected, result))
        self.failIf(result.__name__ in self.public(self._acc),
            "Is public")

_tests.append(TokenListTestCase)


class TokenlistMemberTestCase(TestBase):
    """ svnmailer.settings._accessors.TokenlistMember """
    PREFIX = "TokenlistMember"

    def setUp(self):
        """ Extends test environment """
        super(TokenlistMemberTestCase, self).setUp()
        def mapper(value):
            return "mapped(%s)" % value

        class MyMember(self._acc.TokenlistMember):
            def __init__(this):
                this._allowed = None
                this.CHARSET = 'iso-8859-1'
                this.MAP = None
                this.mapper = mapper

        self._mydescriptor = MyMember

    def test00general(self):
        """ TokenlistMember is a public _base.BasePostmapMember """
        expected = self.mock['svnmailer.settings._base'].BasePostmapMember
        result = self._acc.TokenlistMember

        self.failUnless(issubclass(result, expected),
            "Is not an %r, but %r" % (expected, result))
        self.failUnless(result.__name__ in self.public(self._acc),
            "Is not public")

    def test01init(self):
        """ .init initializes superclass and ._allowed """
        inst = self._acc.TokenlistMember()
        inst.param = {'allowed': (u'a', u'B')}
        inst.init()
        self.mock.result.append(inst._allowed)
        self.expect('BasePostmapMember.init', {u'a': None, u'b': None})

        self.mock.result = []
        inst.param = {}
        inst.init()
        self.mock.result.append(inst._allowed)
        self.expect('BasePostmapMember.init', None)

    def test02doTransform(self):
        """ .doTransform passes TokenLists """
        inst = self._mydescriptor()
        val = self._acc.TokenList((u'a', u'b'))
        self.mock.result.append(inst.doTransform(val))

        self.failUnless(isinstance(self.mock.result[-1], self._acc.TokenList),
            "No tokenlist")
        self.expect(val)

    def test03doTransform(self):
        """ .doTransform splits unicode objects """
        inst = self._mydescriptor()
        self.mock.result.append(inst.doTransform(u' a \nb \t'))

        self.failUnless(isinstance(self.mock.result[-1], self._acc.TokenList),
            "No tokenlist")
        self.expect((u'a', u'b'))

    def test04doTransform(self):
        """ .doTransform decodes string input """
        inst = self._mydescriptor()
        self.mock.result.append(inst.doTransform('\xc9 F'))

        self.failUnless(isinstance(self.mock.result[-1], self._acc.TokenList),
            "No tokenlist")
        self.expect((u'\xc9', u'F'))

    def test05doTransform(self):
        """ .doTransform checks for allowed values """
        class MyMember(self._mydescriptor):
            def _checkallowed(this, value):
                self.mock.result.extend(['_checkallowed', value])
        inst = MyMember()
        inst._allowed = dict.fromkeys(list(u'ab'))
        self.mock.result.append(inst.doTransform(u'a b A  B'))
        inst.MAP = True
        self.mock.result.append(inst.doTransform(u'a b A  B'))

        self.expect(
            '_checkallowed', (u'a', u'b', u'A', u'B'),
            (u'a', u'b', u'A', u'B'),
            (u'a', u'b', u'A', u'B')
        )

    def test06doTransform(self):
        """ .doTransform doesn't checks for allowed values """
        class MyMember(self._mydescriptor):
            def _checkallowed(this, value):
                self.mock.result.extend(['_checkallowed', value])
        inst = MyMember()
        inst._allowed = dict.fromkeys(list(u'ab'))
        inst.MAP = True
        self.mock.result.append(inst.doTransform(u'a b A  B'))

        self.expect((u'a', u'b', u'A', u'B'))

    def test07doTransform(self):
        """ .doTransform passes ValueError """
        class MyMember(self._mydescriptor):
            def _checkallowed(this, value):
                raise ValueError()
        inst = MyMember()
        inst._allowed = dict.fromkeys(list(u'ab'))

        self.failUnlessRaises(ValueError, inst.doTransform, u'a b A  B')

    def test08doTransform(self):
        """ .doTransform rejects invalid types """
        inst = self._mydescriptor()

        self.failUnlessRaises(TypeError, inst.doTransform, self)

    def test09doTransform(self):
        """ .doTransform raises UnicodeError """
        inst = self._mydescriptor()
        inst.CHARSET = "utf-8"

        self.failUnlessRaises(UnicodeError, inst.doTransform, "\xc9")

    def test10doSubstitute(self):
        """ .doSubstitute substitutes each token """
        inst = self._mydescriptor()
        val = self._acc.TokenList((u'%(x)s', u'%(y)s'))
        subst = {u'x': u'foo', u'y': u'bar'}
        self.mock.result.append(inst.doSubstitute(val, subst))

        self.failUnless(isinstance(self.mock.result[-1], self._acc.TokenList),
            "No tokenlist")
        self.expect(
            'util.substitute', u'%(x)s', {u'y': u'bar', u'x': u'foo'},
            'util.substitute', u'%(y)s', {u'y': u'bar', u'x': u'foo'},
            (u'foo', u'bar')
        )

    def test11doPostmap(self):
        """ .doPostmap maps each token """
        inst = self._mydescriptor()
        val = self._acc.TokenList(u'ab')
        self.mock.result.append(inst.doPostmap(val))

        self.failUnless(isinstance(self.mock.result[-1], self._acc.TokenList),
            "No tokenlist")
        self.expect((u'mapped(a)', u'mapped(b)'))

    def test12doPostmap(self):
        """ .doPostmap checks each mapped token """
        class MyMember(self._mydescriptor):
            def _checkallowed(this, value):
                self.mock.result.extend(['_checkallowed', value])
        inst = MyMember()
        val = self._acc.TokenList(u'abAB')
        inst._allowed = dict.fromkeys(list(u'ab'))
        self.mock.result.append(inst.doPostmap(val))

        self.expect(
            '_checkallowed', (u'mapped(a)', u'mapped(b)', u'mapped(A)',
                u'mapped(B)'),
            (u'mapped(a)', u'mapped(b)', u'mapped(A)', u'mapped(B)')
        )

    def test13doPostmap(self):
        """ .doPostmap passes ValueError """
        class MyMember(self._mydescriptor):
            def _checkallowed(this, value):
                raise ValueError()
        inst = MyMember()
        val = self._acc.TokenList(u'abAB')
        inst._allowed = dict.fromkeys(list(u'ab'))
        self.failUnlessRaises(ValueError, inst.doPostmap, val)

    def test14doPostmap(self):
        """ .doPostmap raises UnicodeError """
        def mapper(value):
            return "\xc9"
        inst = self._mydescriptor()
        inst.mapper = mapper
        inst.CHARSET = "utf-8"
        val = self._acc.TokenList(u'abAB')

        self.failUnlessRaises(UnicodeError, inst.doPostmap, val)

    def test15checkallowed(self):
        """ ._checkallowed allows lowercased tokens """
        inst = self._mydescriptor()
        inst._allowed = dict.fromkeys(list(u'ab'))
        val = self._acc.TokenList(u'abAB')
        inst._checkallowed(val)

    def test16checkallowed(self):
        """ ._checkallowed rejects not-allowed lowercased tokens """
        inst = self._mydescriptor()
        inst._allowed = dict.fromkeys(list(u'ab'))

        val = self._acc.TokenList(u'c')
        self.failUnlessRaises(ValueError, inst._checkallowed, val)
        val = self._acc.TokenList(u'C')
        self.failUnlessRaises(ValueError, inst._checkallowed, val)

_tests.append(TokenlistMemberTestCase)


class FilenameMemberTestCase(TestBase):
    """ svnmailer.settings._accessors.FilenameMember """
    PREFIX = "FilenameMember"

    def setUp(self):
        """ Extends the test environment """
        super(FilenameMemberTestCase, self).setUp()
        class MyMember(self._acc.FilenameMember):
            def __init__(this):
                this.CHARSET = 'iso-8859-1'
                this.FILECHARSET = 'utf-8'

        self._mydescriptor = MyMember

    def test00general(self):
        """ FilenameMember is a public _base.BasePremapMember """
        expected = self.mock['svnmailer.settings._base'].BasePremapMember
        result = self._acc.FilenameMember

        self.failUnless(issubclass(result, expected),
            "Is not an %r, but %r" % (expected, result))
        self.failUnless(result.__name__ in self.public(self._acc),
            "Is not public")

    def test01doTransform(self):
        """ .doTransform transforms str and unicode to locale """
        inst = self._mydescriptor()
        self.mock.result.append(inst.doTransform('Andr\xe9'))
        self.failUnless(
            hasattr(self.mock.result[-1], '_already_recoded_filename'),
            "No Filename")
        self.mock.result.append(inst.doTransform(u'Andr\xe9'))
        self.failUnless(
            hasattr(self.mock.result[-1], '_already_recoded_filename'),
            "No Filename")

        self.expect(
            'toLocale', 'Andr\xe9', 'iso-8859-1', 'utf-8', False,
            'locale(Andr\xe9, utf-8)',
            'toLocale', u'Andr\xe9', 'iso-8859-1', 'utf-8', False,
            u'locale(Andr\xe9, utf-8)'
        )

    def test02doTransform(self):
        """ .doTransform passes already recoded file names """
        inst = self._mydescriptor()
        val = inst.doTransform('foo')
        self.mock.result.append(inst.doTransform(val))

        self.failUnless(self.mock.result[-1] is val, "Not identical")
        self.expect('toLocale', 'foo', 'iso-8859-1', 'utf-8', False, val)

    def test03doTransform(self):
        """ .doTransform passes UnicodeError """
        self.mock.error = "util.filename"
        inst = self._mydescriptor()

        self.failUnlessRaises(UnicodeError, inst.doTransform, 'foo')

    def test04doTransform(self):
        """ .doTransform rejects invalid types """
        inst = self._mydescriptor()

        self.failUnlessRaises(TypeError, inst.doTransform, self)

_tests.append(FilenameMemberTestCase)


class CommandlineTestCase(TestBase):
    """ svnmailer.settings._accessors.Commandline """
    PREFIX = "Commandline"

    def test00general(self):
        """ Commandline is a private tuple """
        expected = tuple
        result = self._acc.Commandline

        self.failUnless(issubclass(result, expected),
            "Is not an %r, but %r" % (expected, result))
        self.failIf(result.__name__ in self.public(self._acc),
            "Is public")

    def test01new(self):
        """ .__new__ returns None for empty or None command """
        self.mock.result.append(self._acc.Commandline('', None, None))
        self.mock.result.append(self._acc.Commandline(None, None, None))

        self.expect(None, None)

    def test02new(self):
        """ .__new__ initializes with a splitted command line """
        self.mock.result.append(self._acc.Commandline(
            u'Andr\xe9 Malo', 'iso-8859-1', 'utf-8'
        ))

        self.failUnless(
            isinstance(self.mock.result[-1], self._acc.Commandline),
            "No command line")
        self.expect(
            'toLocale', u'Andr\xe9', 'iso-8859-1', 'utf-8', False,
            (u'locale(Andr\xe9, utf-8)', u'Malo')
        )

    def test03new(self):
        """ .__new__ transforms arguments to unicode """
        self.mock.result.append(self._acc.Commandline(
            'Malo Andr\xe9', 'iso-8859-1', 'utf-8'
        ))

        self.failUnless(
            isinstance(self.mock.result[-1], self._acc.Commandline),
            "No command line")
        self.expect(
            'toLocale', 'Malo', 'iso-8859-1', 'utf-8', False,
            ('locale(Malo, utf-8)', u'Andr\xe9')
        )

    def test04new(self):
        """ .__new__ raises UnicodeError """
        self.failUnlessRaises(UnicodeError,
            self._acc.Commandline, 'Malo Andr\xe9', 'utf-8', 'utf-8')

    def test05new(self):
        """ .__new__ passes UnicodeError """
        self.mock.error = 'util.filename'
        self.failUnlessRaises(UnicodeError,
            self._acc.Commandline, u'Andr\xe9 Malo', 'iso-8859-1', 'utf-8')

    def test06new(self):
        """ .__new__ passes ValueError """
        self.mock.error = 'util.splitCommand'
        self.failUnlessRaises(ValueError,
            self._acc.Commandline, u'Andr\xe9 Malo', 'iso-8859-1', 'utf-8')

_tests.append(CommandlineTestCase)


class CommandlineMemberTestCase(TestBase):
    """ svnmailer.settings._accessors.CommandlineMember """
    PREFIX = "CommandlineMember"

    def setUp(self):
        """ Extends test environment """
        super(CommandlineMemberTestCase, self).setUp()
        class Commandline(object):
            def __init__(this, value, charset, fcharset):
                if self.mock.error == 'value':
                    raise ValueError()
                elif self.mock.error == 'unicode':
                    raise UnicodeError()
                self.mock.result.extend(['Commandline',
                    value, charset, fcharset])
                this.__mocked__ = '*Commandline*'

        class MyMember(self._acc.CommandlineMember):
            def __init__(this):
                this.CHARSET = 'iso-8859-1'
                this.FILECHARSET = 'utf-8'

        self._acc.Commandline = Commandline
        self._mydescriptor = MyMember

    def test00general(self):
        """ CommandlineMember is a public _base.BasePremapMember """
        expected = self.mock['svnmailer.settings._base'].BasePremapMember
        result = self._acc.CommandlineMember

        self.failUnless(issubclass(result, expected),
            "Is not an %r, but %r" % (expected, result))
        self.failUnless(result.__name__ in self.public(self._acc),
            "Is not public")

    def test01doTransform(self):
        """ .doTransform transforms str and unicode to Commandline """
        self.mock.result.append(
            self._mydescriptor().doTransform('Andr\xe9').__mocked__
        )
        self.mock.result.append(
            self._mydescriptor().doTransform(u'Andr\xe9').__mocked__
        )

        self.expect(
            'Commandline', 'Andr\xe9', 'iso-8859-1', 'utf-8',
            '*Commandline*',
            'Commandline', u'Andr\xe9', 'iso-8859-1', 'utf-8',
            '*Commandline*'
        )

    def test02doTransform(self):
        """ .doTransform passes Commandline instances """
        val = self._acc.Commandline('foo', 'iso-8859-2', 'utf-7')
        self.mock.result.append(self._mydescriptor().doTransform(val))

        self.failUnless(self.mock.result[-1] is val, "Not identical")
        self.expect('Commandline', 'foo', 'iso-8859-2', 'utf-7', val)

    def test03doTransform(self):
        """ .doTransform rejects invalid types """
        self.failUnlessRaises(TypeError,
            self._mydescriptor().doTransform, self)

    def test04doTransform(self):
        """ .doTransform passes UnicodeError """
        self.mock.error = "unicode"
        self.failUnlessRaises(UnicodeError,
            self._mydescriptor().doTransform, 'foo')

    def test05doTransform(self):
        """ .doTransform passes ValueError """
        self.mock.error = "value"
        self.failUnlessRaises(ValueError,
            self._mydescriptor().doTransform, 'foo')

_tests.append(CommandlineMemberTestCase)


class QuotedStringTestCase(TestBase):
    """ svnmailer.settings._accessors.QuotedString """
    PREFIX = "QuotedString"

    def test00general(self):
        """ QuotedString is a private str """
        expected = str
        result = self._acc.QuotedString

        self.failUnless(issubclass(result, expected),
            "Is not an %r, but %r" % (expected, result))
        self.failIf(result.__name__ in self.public(self._acc),
            "Is public")

    def test01new(self):
        """ .__new__ unquotes strings properly """
        tests = (r'', r'""', r'bar', r'\"bar', r'"abc\\"', r'"woo \"!\\"')

        for test in tests:
            self.mock.result.append(self._acc.QuotedString(test))

        rmatch = r'(?:[^"\s]\S*|"[^\\"]*(?:\\[\\"][^\\"]*)*")$'
        rsub = r'\\([\\"])'
        self.expect(
            '',
            'Regex.match', rmatch, '""',
            'Regex.sub', rsub, r'\1', '',
            '',
            'Regex.match', rmatch, 'bar',
            'bar',
            'Regex.match', rmatch, r'\"bar',
            '\\"bar',
            'Regex.match', rmatch, r'"abc\\"',
            'Regex.sub', rsub, r'\1', 'abc\\\\',
            'abc\\',
            'Regex.match', rmatch, r'"woo \"!\\"',
            'Regex.sub', rsub, r'\1', 'woo \\"!\\\\',
            'woo "!\\'
        )

    def test02new(self):
        """ .__new__ rejects invalid input """
        tests = (r' bar', r'"\"bar', r'"abc\\" ', r'"woo " ')

        for test in tests:
            self.failUnlessRaises(ValueError, self._acc.QuotedString, test)

    def test03repr(self):
        """ .__repr__ returns the proper representation """
        val = self._acc.QuotedString(r'"woo \"!\\"')
        self.mock.result = [repr(val)]

        self.expect(repr(r'"woo \"!\\"'))

_tests.append(QuotedStringTestCase)


class QuotedstringMemberTestCase(TestBase):
    """ svnmailer.settings._accessors.QuotedstringMember """
    PREFIX = "QuotedstringMember"

    def setUp(self):
        """ Extends the test environment """
        super(QuotedstringMemberTestCase, self).setUp()
        class QuotedString(object):
            def __init__(this, value):
                if self.mock.error == 'value':
                    raise ValueError()
                self.mock.result.extend(['QuotedString', value])
                this.__mocked__ = "*QuotedString*"

        self._acc.QuotedString = QuotedString

    def test00general(self):
        """ QuotedStringMember is a public _base.BasePremapMember """
        expected = self.mock['svnmailer.settings._base'].BasePremapMember
        result = self._acc.QuotedstringMember

        self.failUnless(issubclass(result, expected),
            "Is not an %r, but %r" % (expected, result))
        self.failUnless(result.__name__ in self.public(self._acc),
            "Is not public")

    def test01doTransform(self):
        """ .doTransform transforms string input to QuotedString """
        self.mock.result.append(
            self._acc.QuotedstringMember().doTransform('foo').__mocked__
        )

        self.expect('QuotedString', 'foo', '*QuotedString*')

    def test02doTransform(self):
        """ .doTransform passes QuotedString input """
        val = self._acc.QuotedString('bar')
        self.mock.result.append(
            self._acc.QuotedstringMember().doTransform(val)
        )

        self.expect('QuotedString', 'bar', val)

    def test03doTransform(self):
        """ .doTransform rejects invalid types """
        self.failUnlessRaises(TypeError,
            self._acc.QuotedstringMember().doTransform, self)
        self.failUnlessRaises(TypeError,
            self._acc.QuotedstringMember().doTransform, u"foo")

    def test04doTransform(self):
        """ .doTransform passes ValueError """
        self.mock.error = 'value'
        self.failUnlessRaises(ValueError,
            self._acc.QuotedstringMember().doTransform, "foo")

_tests.append(QuotedstringMemberTestCase)


class StdinMemberTestCase(TestBase):
    """ svnmailer.settings._accessors.StdinMember """
    PREFIX = "StdinMember"

    def test00general(self):
        """ StdinMember is a public _base.BaseMember """
        expected = self.mock['svnmailer.settings._base'].BaseMember
        result = self._acc.StdinMember

        self.failUnless(issubclass(result, expected),
            "Is not an %r, but %r" % (expected, result))
        self.failUnless(result.__name__ in self.public(self._acc),
            "Is not public")

    def test01initial(self):
        """ ._stdin is properly filled initially """
        self.mock.result = [self._acc.StdinMember._stdin]

        self.expect(None)

    def test02substitute(self):
        """ .substitute reads STDIN once and returns it as string """
        import cStringIO, sys
        fp = cStringIO.StringIO('foooo')
        fp2 = cStringIO.StringIO('baaaar')
        inst = self._acc.StdinMember()

        old_fp = sys.stdin
        try:
            sys.stdin = fp
            self.mock.result.append(inst.substitute(None, None))
            sys.stdin = fp2
            self.mock.result.append(inst.substitute(None, None))
        finally:
            sys.stdin = old_fp

        self.expect('foooo', 'foooo')

_tests.append(StdinMemberTestCase)


class MailActionTestCase(TestBase):
    """ svnmailer.settings._accessors.MailAction """
    PREFIX = "MailAction"

    def setUp(self):
        """ Extends the test environment """
        super(MailActionTestCase, self).setUp()
        class MyAction(self._acc.MailAction):
            def __new__(cls, action):
                return object.__new__(cls)

        self._myaction = MyAction

    def test00general(self):
        """ MailAction is a private object """
        expected = object
        result = self._acc.MailAction

        self.failUnless(issubclass(result, expected),
            "Is not an %r, but %r" % (expected, result))
        self.failIf(result.__name__ in self.public(self._acc),
            "Is public")

    def test01initial(self):
        """ MailAction's class attributes are properly filled initially """
        cls = self._acc.MailAction
        self.mock.result = [
            cls.maxbytes, cls.mode, cls.truncate, cls.drop, cls.scope,
            cls.TRUNCATE, cls.URLS, cls.SPLIT, cls.REVPROP, cls.LOCKS
        ]

        self.expect(
            0, None, False, None, (),
            u'truncate', u'showurls', u'split', u'revprop-changes', u'locks'
        )

    def test02new(self):
        """ .__new__ returns None on empty or None input """
        class MyAction(self._acc.MailAction):
            def __init__(self, action):
                pass
        self.mock.result.append(MyAction(''))
        self.mock.result.append(MyAction(None))

        self.expect(None, None)

    def test03new(self):
        """ .__new__ returns MailAction on true input """
        class MyAction(self._acc.MailAction):
            def __init__(this, action):
                self.mock.result.append('__init__')
        inst = MyAction('foo')

        self.failUnless(isinstance(inst, self._acc.MailAction),
            "No MailAction")
        self.expect('__init__')

    def test04init(self):
        """ .__init__ rejects input with less than 2 tokens """
        for test in (" ", " a  \t", "10"):
            self.failUnlessRaises(ValueError, self._myaction, test)

    def test05init(self):
        """ .__init__ rejects non-int size limit """
        for test in ("a split", "0.0 split", "0xa split"):
            self.failUnlessRaises(ValueError, self._myaction, test)

    def test06init(self):
        """ .__init__ accepts int size limit """
        self.mock.result.append(self._myaction('10 split').maxbytes)

        self.expect(10)

    def test07init(self):
        """ .__init__ rejects invalid mode tokens """
        self.failUnlessRaises(ValueError, self._myaction, "10 foo")

    def test08init(self):
        """ .__init__ accepts valid mode tokens """
        tests = ("10 Truncate", "11 truncate", "12 SPLIT", "13 split",
            "14 showurls", "15 ShowUrls")
        for test in tests:
            inst = self._myaction(test)
            self.mock.result.extend([
                inst.maxbytes, inst.mode, inst.truncate, inst.drop, inst.scope
            ])

        self.expect(
            10, 'truncate', False, None, (), 11, 'truncate', False, None, (),
            12, 'split', False, None, (),    13, 'split', False, None, (),
            14, 'showurls', False, None, (), 15, 'showurls', False, None, ()
        )

    def test09init(self):
        """ .__init__ rejects invalid /truncate modifiers """
        self.failUnlessRaises(ValueError,
            self._myaction, "10 truncate/truncate")

    def test10init(self):
        """ .__init__ accepts valid /truncate modifiers """
        for test in ("10 showurls/truncate", "11 split/Truncate"):
            inst = self._myaction(test)
            self.mock.result.extend([
                inst.maxbytes, inst.mode, inst.truncate, inst.drop, inst.scope
            ])

        self.expect(
            10, 'showurls', True, None, (),
            11, 'split', True, None, ()
        )

    def test11init(self):
        """ .__init__ rejects invalid /<drop> modifiers """
        tests = (
            "10 showurls/11", "12 truncate/13", "14 split/foo",
            "15 showurls/truncate/16", "17 split/truncate/bar",
        )
        for test in tests:
            self.failUnlessRaises(ValueError, self._myaction, test)

    def test12init(self):
        """ .__init__ accepts valid /<drop> modifiers """
        for test in ("10 split/truncate/11", "12 split/13"):
            inst = self._myaction(test)
            self.mock.result.extend([
                inst.maxbytes, inst.mode, inst.truncate, inst.drop, inst.scope
            ])

        self.expect(10, 'split', True, 11, (), 12, 'split', False, 13, ())

    def test13init(self):
        """ .__init__ rejects too many modifier tokens """
        tests = (
            "10 split/11/12", "13 split/14/foo",
            "15 split/truncate/16/17", "18 split/truncate/19/bar",
        )
        for test in tests:
            self.failUnlessRaises(ValueError, self._myaction, test)

    def test14init(self):
        """ .__init__ rejects invalid scope tokens """
        for test in ("10 split foo", "11 split locks foo"):
            self.failUnlessRaises(ValueError, self._myaction, test)

    def test15init(self):
        """ .__init__ accepts valid scope tokens """
        tests = ("10 split/truncate/11 revprop-changes", "12 split/13 locks",
            "14 split/truncate/15 locks revprop-changes")
        for test in tests:
            inst = self._myaction(test)
            self.mock.result.extend([
                inst.maxbytes, inst.mode, inst.truncate, inst.drop,
                dict.fromkeys(inst.scope)
            ])

        self.expect(
            10, 'split', True, 11, {'revprop-changes': None},
            12, 'split', False, 13, {'locks': None},
            14, 'split', True, 15, {'locks': None, 'revprop-changes': None}
        )

    def test16repr(self):
        """ .__repr__ returns the proper representation """
        tests = ("\t1  TRUNCATE", "2 ShowUrls/Truncate", "3 SPLIT/Truncate/4",
            "5 Split/6", "7 Truncate Locks Revprop-Changes",
            "8 split/TrunCate/9 LOCKS")
        for test in tests:
            self.mock.result.append(repr(self._myaction(test)))

        self.expect(
            '1 truncate', '2 showurls/truncate', '3 split/truncate/4',
                '5 split/6', '7 truncate locks revprop-changes',
                '8 split/truncate/9 locks'
        )

_tests.append(MailActionTestCase)


class MailactionMemberTestCase(TestBase):
    """ svnmailer.settings._accessors.MailactionMember """
    PREFIX = "MailactionMember"

    def setUp(self):
        """ Extends test environment """
        super(MailactionMemberTestCase, self).setUp()
        class MailAction(object):
            def __init__(this, value):
                if self.mock.error == 'value':
                    raise ValueError()
                self.mock.result.extend(['MailAction', value])
                this.__mocked__ = '*MailAction*'

        class MyMember(self._acc.MailactionMember):
            def __init__(this):
                this.CHARSET = "iso-8859-1"

        self._acc.MailAction = MailAction
        self._mydescriptor = MyMember

    def test00general(self):
        """ MailactionMember is a public _base.BasePremapMember """
        expected = self.mock['svnmailer.settings._base'].BasePremapMember
        result = self._acc.MailactionMember

        self.failUnless(issubclass(result, expected),
            "Is not an %r, but %r" % (expected, result))
        self.failUnless(result.__name__ in self.public(self._acc),
            "Is not public")

    def test01doTransform(self):
        """ .doTransform converts str and unicode to MailAction """
        inst = self._mydescriptor()
        self.mock.result.append(inst.doTransform('foo').__mocked__)
        self.mock.result.append(inst.doTransform(u'foo').__mocked__)

        self.expect(
            'MailAction', u'foo',
            '*MailAction*',
            'MailAction', u'foo',
            '*MailAction*'
        )

    def test02doTransform(self):
        """ .doTransform passes MailAction instances """
        val = self._acc.MailAction('foo')
        self.mock.result.append(self._mydescriptor().doTransform(val))

        self.failUnless(self.mock.result[-1] is val, "Not identical")
        self.expect('MailAction', 'foo', val)

    def test03doTransform(self):
        """ .doTransform rejects invalid types """
        self.failUnlessRaises(TypeError,
            self._mydescriptor().doTransform, self)

    def test04doTransform(self):
        """ .doTransform raises UnicodeError """
        inst = self._mydescriptor()
        inst.CHARSET = 'utf-8'

        self.failUnlessRaises(UnicodeError, inst.doTransform, 'Andr\xe9')

    def test05doTransform(self):
        """ .doTransform passes ValueError """
        self.mock.error = "value"

        self.failUnlessRaises(ValueError,
            self._mydescriptor().doTransform, 'foo')

_tests.append(MailactionMemberTestCase)


#### Glue code                                      ####
########################################################
__all__.extend(_tests)

def getSuite():
    """ Returns the test suite for this module """
    return [_test.__name__ for _test in _tests]
