# -*- coding: iso-8859-1 -*-
# pylint: disable-msg=W0121,W0612,C0103,W0703
# pylint-version = 0.7.0
#
# Copyright 2004-2005 Andr Malo or his licensors, as applicable
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""
Testcases for svnmailer.subversion

:var _tests: The test classes
:type _tests: ``list``
"""
__author__    = "Andr Malo"
__docformat__ = "restructuredtext en"
__all__       = ['getSuite']

import unittest
_tests = []
_xtests = []
_repos_path = None


class importTestCase(unittest.TestCase):
    """ Importing svnmailer.subversion """

    def testImport(self):
        """ from svnmailer import subversion """
        try:
            from svnmailer import subversion
        except Exception, exc:
            self.fail(
                "svnmailer.subversion could not be imported: %s" % str(exc)
            )

_tests.append(importTestCase)


class subversionTestBase(unittest.TestCase):
    """ base class for subversion tests """

    def setUp(self):
        """ import the module """
        from svnmailer import subversion
        self._subversion = subversion


#### Optional Tests ####

class RepositoryTestCase(subversionTestBase):
    """ svnmailer.subversion.Repository """

    def test00open(self):
        """ Repository class is able to open a repos """
        repos = self._subversion.Repository(_repos_path)

    def test01getChangesList(self):
        """ getChangesList returns a proper list of changes """
        repos = self._subversion.Repository(_repos_path)

        clist = repos.getChangesList(1)
        self.failUnless(
            len(clist) == 1
            and clist[0].path == 'foo'
            and clist[0].isDirectory()
            and clist[0].wasAdded(),
            "The returned list is wrong, found %r" % clist
        )

        clist = repos.getChangesList(2)
        self.failUnless(
            len(clist) == 1
            and clist[0].path == 'foo'
            and clist[0].isDirectory()
            and clist[0].wasModified()
            and clist[0].hasPropertyChanges(),
            "The returned list is wrong, found %r" % clist
        )

        clist = repos.getChangesList(5)
        self.failUnless(
            len(clist) == 1
            and clist[0].path == 'foo/eggs'
            and not clist[0].isDirectory()
            and clist[0].wasCopied()
            and clist[0].hasContentChanges()
            and clist[0].hasPropertyChanges()
            and clist[0].getModifiedProperties() == {'color': (None, "white")},
            "The returned list is wrong, found %r" % clist
        )

    def test02getPathProperties(self):
        """ getPathProperties returns the correct properties """
        repos = self._subversion.Repository(_repos_path)

        prop = repos.getPathProperties("/foo", 1)
        self.failUnlessEqual(
            prop, {},
            "The returned dict is wrong, found %r" % prop
        )

        prop = repos.getPathProperties("/foo", 2)
        self.failUnlessEqual(
            prop, {"someprop": "somevalue"},
            "The returned dict is wrong, found %r" % prop
        )

    def test03getPathProperty(self):
        """ getPathProperty returns the value or None """
        repos = self._subversion.Repository(_repos_path)

        prop = repos.getPathProperty("someprop", "/foo", 1)
        self.failUnlessEqual(
            prop, None,
            "Unknown property doesn't return None, but %r" % prop
        )

        prop = repos.getPathProperty("someprop", "/foo", 2)
        self.failUnlessEqual(
            prop, "somevalue",
            "property value returned was %r" % prop
        )

    def test04getPathMimeType(self):
        """ getPathMimeType returns the value or None """
        repos = self._subversion.Repository(_repos_path)

        prop = repos.getPathMimeType("/foo/spam", 3)
        self.failUnlessEqual(
            prop, None,
            "Unset mime type doesn't return None, but %r" % prop
        )

        prop = repos.getPathMimeType("/foo/and.txt", 4)
        self.failUnlessEqual(
            prop, "text/plain",
            "Mime type returned was %r" % prop
        )

    def test05dumpPathContent(self):
        """ dumpPathContent dumps the correct data """
        import cStringIO
        tests = (
            (3, "/foo/spam", "This is spam.\n"),
            (4, "foo/and.txt", "This is and.txt\n"),
        )

        repos = self._subversion.Repository(_repos_path)
        for rev, path, expected in tests:
            fp = cStringIO.StringIO()
            repos.dumpPathContent(fp, path, rev)
            result = fp.getvalue()
            fp.close()
            self.failUnlessEqual(
                result, expected,
                "Unexpected data in rev %d, found %r, expected %r" %
                (rev, result, expected)
            )

    def test06getRevisionTime(self):
        """ getRevisionTime returns the proper time """
        import time
        tests = (
            (1, 1104881023), # 2005-01-05 00:23:43 +0100
            (2, 1104881088), # 2005-01-05 00:24:48 +0100
            (3, 1104958527), # 2005-01-05 21:55:27 +0100
            (4, 1104958970), # 2005-01-05 22:02:50 +0100
        )

        repos = self._subversion.Repository(_repos_path)
        for rev, expected in tests:
            result = repos.getRevisionTime(rev)
            self.failUnlessEqual(
                expected, result,
                "Wrong time returned for rev %d: %d (%s)" %
                (rev, result, time.ctime(result))
            )

    def test07getRevisionAuthor(self):
        """ getRevisionAuthor returns the correct author """
        repos = self._subversion.Repository(_repos_path)
        tests = (
            (1, 'nd'),
            (3, 'dn'),
        )

        for rev, expected in tests:
            result = repos.getRevisionAuthor(rev)
            self.failUnlessEqual(
                expected, result,
                "Wrong author returned for rev %d: %r " % (rev, result)
            )

    def test08getRevisionLog(self):
        """ getRevisionLog returns the correct log entry """
        repos = self._subversion.Repository(_repos_path)
        tests = (
            (2, 'set property someprop to somevalue on foo\n'),
            (4, 'this is text/plain\n'),
        )

        for rev, expected in tests:
            result = repos.getRevisionLog(rev)
            self.failUnlessEqual(
                expected, result,
                "Wrong author returned for rev %d: %r " % (rev, result)
            )

_xtests.append(RepositoryTestCase)


#### Glue code                                      ####
########################################################
__all__.extend(_xtests)

class DummyTestCase(unittest.TestCase):
    """ skipping real tests """
    def testNothing(self):
        """ (could not find test repos) repository tests SKIPPED """
        pass

def findRepos():
    """ Finds the test repository

        :return: Repospath or None
        :rtype: ``str``
    """
    import os
    repos_path = os.environ.get('TESTREPOSITORY')
    if repos_path and os.path.isdir(repos_path):
        return repos_path

    return None

def getSuite():
    """ Returns the test suite for this module """
    global _repos_path
    _repos_path = findRepos()

    tests = _tests[:]
    if _repos_path:
        tests.extend(_xtests)
    else:
        tests.append(DummyTestCase)

    return [_test.__name__ for _test in tests]
